@@ -1,5 +1,3 @@
-/* $XdotOrg: app/xdm/session.c,v 1.7 2006-06-03 00:05:24 alanc Exp $ */
-/* $Xorg: session.c,v 1.8 2001/02/09 02:05:40 xorgcvs Exp $ */
 /*
 
 Copyright 1988, 1998  The Open Group
@@ -27,7 +25,6 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/xdm/session.c,v 3.36 2003/09/29 21:00:08 herrb Exp $ */
 
 /*
  * xdm - display manager daemon
@@ -36,6 +33,10 @@ from The Open Group.
  * session.c
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "dm.h"
 #include "dm_auth.h"
 #include "dm_error.h"
@@ -54,22 +55,27 @@ from The Open Group.
 #endif
 
 #ifndef USE_PAM        /* PAM modules should handle these */
-#ifdef SECURE_RPC
-# include <rpc/rpc.h>
-# include <rpc/key_prot.h>
-# if !HAVE_DECL_KEY_SETNET
+# ifdef SECURE_RPC
+#  include <rpc/rpc.h>
+#  include <rpc/key_prot.h>
+#  if !HAVE_DECL_KEY_SETNET
 extern int key_setnet(struct key_netstarg *arg);
+#  endif
+# endif
+# ifdef K5AUTH
+#  include <krb5/krb5.h>
 # endif
-#endif
-#ifdef K5AUTH
-# include <krb5/krb5.h>
-#endif
 #endif /* USE_PAM */
 
 #ifdef __SCO__
-#include <prot.h>
+# include <prot.h>
 #endif
 
+#ifdef HAVE_SELINUX
+#include <selinux/selinux.h>
+#include <selinux/get_context_list.h>
+#endif /* HAVE_SELINUX */
+
 #ifndef GREET_USER_STATIC
 # include <dlfcn.h>
 # ifndef RTLD_NOW
@@ -77,6 +83,42 @@ extern int key_setnet(struct key_netstarg *arg);
 # endif
 #endif
 
+#ifdef HAVE_SELINUX
+/* This should be run just before we exec the user session. */
+static int
+xdm_selinux_setup (const char *login)
+  {
+	security_context_t scontext;
+	int ret = -1;
+	char *seuser=NULL;
+	char *level=NULL;
+
+	/* If SELinux is not enabled, then we don't do anything. */
+	if ( is_selinux_enabled () <= 0)
+		return TRUE;
+
+	if (getseuserbyname(login, &seuser, &level) == 0) {
+		ret=get_default_context_with_level(seuser, level, 0, &scontext);
+		free(seuser);
+		free(level);
+	}
+	if (ret < 0 || scontext == NULL) {
+		LogError ("SELinux: unable to obtain default security context for %s\n", login);
+		return FALSE;
+	}
+
+	if (setexeccon (scontext) != 0) {
+	freecon (scontext);
+	LogError ("SELinux: unable to set executable context %s\n",
+	      (char *)scontext);
+	return FALSE;
+	}
+
+	freecon (scontext);
+	return TRUE;
+}
+#endif /* HAVE_SELINUX */
+
 static	int	runAndWait (char **args, char **environ);
 
 #ifdef HAVE_GRP_H
@@ -609,25 +651,29 @@ StartClient (
 #endif
 
 #ifndef AIXV3
-#ifndef HAS_SETUSERCONTEXT
-	if (setgid(verify->gid) < 0) {
-	    LogError ("setgid %d (user \"%s\") failed, errno=%d\n",
-		     verify->gid, name, errno);
+# ifndef HAS_SETUSERCONTEXT
+	if (setgid (verify->gid) < 0) {
+	    LogError ("setgid %d (user \"%s\") failed: %s\n",
+		      verify->gid, name, _SysErrorMsg (errno));
 	    return (0);
 	}
-#if defined(BSD) && (BSD >= 199103)
-	if (setlogin(name) < 0) {
-	    LogError ("setlogin for \"%s\" failed, errno=%d", name, errno);
-	    return(0);
-	}
-#endif
-#ifndef QNX4
-	if (initgroups(name, verify->gid) < 0) {
-	    LogError ("initgroups for \"%s\" failed, errno=%d\n", name, errno);
+#  if defined(BSD) && (BSD >= 199103)
+	if (setlogin (name) < 0) {
+	    LogError ("setlogin for \"%s\" failed: %s\n",
+		      name, _SysErrorMsg (errno));
 	    return (0);
 	}
-#endif   /* QNX4 doesn't support multi-groups, no initgroups() */
-#ifdef USE_PAM
+#  endif
+#  ifndef QNX4
+	if (initgroups (name, verify->gid) < 0) {
+	    LogError ("initgroups for \"%s\" failed: %s\n",
+		      name, _SysErrorMsg (errno));
+	    return (0);
+	}
+#  endif   /* QNX4 doesn't support multi-groups, no initgroups() */
+# endif /* !HAS_SETUSERCONTEXT */
+
+# ifdef USE_PAM
 	if (pamh) {
 	    long i;
 	    char **pam_env;
@@ -646,13 +692,15 @@ StartClient (
 	    }
 
 	}
-#endif
+# endif
+
+# ifndef HAS_SETUSERCONTEXT
 	if (setuid(verify->uid) < 0) {
-	    LogError ("setuid %d (user \"%s\") failed, errno=%d\n",
-		     verify->uid, name, errno);
+	    LogError ("setuid %d (user \"%s\") failed: %s\n",
+		      verify->uid, name, _SysErrorMsg (errno));
 	    return (0);
 	}
-#else /* HAS_SETUSERCONTEXT */
+# else /* HAS_SETUSERCONTEXT */
 	/*
 	 * Set the user's credentials: uid, gid, groups,
 	 * environment variables, resource limits, and umask.
@@ -660,23 +708,25 @@ StartClient (
 	pwd = getpwnam(name);
 	if (pwd) {
 	    if (setusercontext(NULL, pwd, pwd->pw_uid, LOGIN_SETALL) < 0) {
-		LogError ("setusercontext for \"%s\" failed, errno=%d\n", name,
-		    errno);
+		LogError ("setusercontext for \"%s\" failed: %s\n",
+			  name, _SysErrorMsg (errno));
 		return (0);
 	    }
 	    endpwent();
 	} else {
-	    LogError ("getpwnam for \"%s\" failed, errno=%d\n", name, errno);
+	    LogError ("getpwnam for \"%s\" failed: %s\n",
+		      name, _SysErrorMsg (errno));
 	    return (0);
 	}
-#endif /* HAS_SETUSERCONTEXT */
+# endif /* HAS_SETUSERCONTEXT */
 #else /* AIXV3 */
 	/*
 	 * Set the user's credentials: uid, gid, groups,
 	 * audit classes, user limits, and umask.
 	 */
 	if (setpcred(name, NULL) == -1) {
-	    LogError ("setpcred for \"%s\" failed, errno=%d\n", name, errno);
+	    LogError ("setpcred for \"%s\" failed: %s\n",
+		      name, _SysErrorMsg (errno));
 	    return (0);
 	}
 #endif /* AIXV3 */
@@ -686,7 +736,7 @@ StartClient (
 	 * for user-based authorization schemes,
 	 * use the password to get the user's credentials.
 	 */
-#ifdef SECURE_RPC
+# ifdef SECURE_RPC
 	/* do like "keylogin" program */
 	{
 	    char    netname[MAXNETNAMELEN+1], secretkey[HEXKEYBYTES+1];
@@ -709,7 +759,7 @@ StartClient (
             if (key_setnet(&netst) < 0) {
 		Debug ("Could not set secret key.\n");
             }
-	    free(netst.st_netname);	
+	    free(netst.st_netname);
 	    /* is there a key, and do we have the right password? */
 	    if (keyret == 1) {
 		if (*secretkey) {
@@ -740,8 +790,8 @@ StartClient (
 	    }
 	    bzero(secretkey, strlen(secretkey));
 	}
-#endif
-#ifdef K5AUTH
+# endif
+# ifdef K5AUTH
 	/* do like "kinit" program */
 	{
 	    int i, j;
@@ -767,12 +817,27 @@ StartClient (
 		}
 	    }
 	}
-#endif /* K5AUTH */
+# endif /* K5AUTH */
 #endif /* !USE_PAM */
+
+	if (d->windowPath)
+		verify->userEnviron = setEnv(verify->userEnviron, "WINDOWPATH", d->windowPath);
+
 	if (passwd != NULL)
 	    bzero(passwd, strlen(passwd));
 
 	SetUserAuthorization (d, verify);
+#ifdef HAVE_SELINUX
+   /*
+    * For Security Enhanced Linux:
+    * set the default security context for this user.
+    */
+   if ( ! xdm_selinux_setup (name)) {
+      LogError ("failed to set security context\n");
+       exit (UNMANAGE_DISPLAY);
+       return (0);
+   }
+#endif /* HAVE_SELINUX */
 	home = getEnv (verify->userEnviron, "HOME");
 	if (home)
 	    if (chdir (home) == -1) {
@@ -782,7 +847,7 @@ StartClient (
 		verify->userEnviron = setEnv(verify->userEnviron, "HOME", "/");
 	    }
 	if (verify->argv) {
-		Debug ("executing session %s\n", verify->argv[0]);
+		LogInfo ("executing session %s\n", verify->argv[0]);
 		execute (verify->argv, verify->userEnviron);
 		LogError ("Session \"%s\" execution failed (err %d)\n", verify->argv[0], errno);
 	} else {
@@ -796,8 +861,8 @@ StartClient (
 	if (passwd != NULL)
 	    bzero(passwd, strlen(passwd));
 	Debug ("StartSession, fork failed\n");
-	LogError ("can't start session on \"%s\", fork failed, errno=%d\n",
-		  d->name, errno);
+	LogError ("can't start session on \"%s\", fork failed: %s\n",
+		  d->name, _SysErrorMsg (errno));
 	return 0;
     default:
 	if (passwd != NULL)
@@ -812,21 +877,28 @@ int
 source (char **environ, char *file)
 {
     char	**args, *args_safe[2];
-    int		ret;
+    int		ret = 0;
+    FILE	*f;
 
     if (file && file[0]) {
-	Debug ("source %s\n", file);
-	args = parseArgs ((char **) 0, file);
-	if (!args) {
-	    args = args_safe;
-	    args[0] = file;
-	    args[1] = NULL;
+	f = fopen (file, "r");
+	if (!f)
+	    LogInfo ("not sourcing %s (%s)\n", file, _SysErrorMsg (errno));
+	else {
+	    fclose (f);
+	    LogInfo ("sourcing %s\n", file);
+	    args = parseArgs ((char **) 0, file);
+	    if (!args) {
+		args = args_safe;
+		args[0] = file;
+		args[1] = NULL;
+	    }
+	    ret = runAndWait (args, environ);
+	    freeArgs (args);
 	}
-	ret = runAndWait (args, environ);
-	freeArgs (args);
-	return ret;
-    }
-    return 0;
+    } else
+	Debug ("source() given null pointer in file argument\n");
+    return ret;
 }
 
 static int
@@ -838,6 +910,10 @@ runAndWait (char **args, char **environ)
     switch (pid = fork ()) {
     case 0:
 	CleanUpChild ();
+#ifdef XDMCP
+	/* The chooser socket is not closed by CleanUpChild() */
+	DestroyWellKnownSockets();
+#endif
 	execute (args, environ);
 	LogError ("can't execute \"%s\" (err %d)\n", args[0], errno);
 	exit (1);
@@ -857,18 +933,22 @@ runAndWait (char **args, char **environ)
 void
 execute (char **argv, char **environ)
 {
+    int err;
     /* give /dev/null as stdin */
     (void) close (0);
     open ("/dev/null", O_RDONLY);
     /* make stdout follow stderr to the log file */
     dup2 (2,1);
+    Debug ("attempting to execve() %s\n", argv[0]);
     execve (argv[0], argv, environ);
+    err = errno;
+    Debug ("execve() of %s failed: %s\n", argv[0], _SysErrorMsg (errno));
     /*
      * In case this is a shell script which hasn't been
      * made executable (or this is a SYSV box), do
      * a reasonable thing
      */
-    if (errno != ENOENT) {
+    if (err != ENOENT) {
 	char	program[1024], *e, *p, *optarg;
 	FILE	*f;
 	char	**newargv, **av;
@@ -926,6 +1006,7 @@ execute (char **argv, char **environ)
 	while ((*av++ = *argv++))
 	    /* SUPPRESS 530 */
 	    ;
+	Debug ("Attempting to execve() %s\n", newargv[0]);
 	execve (newargv[0], newargv, environ);
     }
 }
@@ -961,12 +1042,7 @@ systemEnv (struct display *d, char *user, char *home)
     env = setEnv (env, "SHELL", d->systemShell);
     if (d->authFile)
 	    env = setEnv (env, "XAUTHORITY", d->authFile);
+    if (d->windowPath)
+	    env = setEnv (env, "WINDOWPATH", d->windowPath);
     return env;
 }
-
-#if (defined(Lynx) && !defined(HAS_CRYPT))
-char *crypt(char *s1, char *s2)
-{
-	return(s2);
-}
-#endif
